<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;

use Financeiro\Entities\Cnabpag;
use Fiscal\Entities\Boletpagar;
use Geral\Entities\Empresa;

class LeitorCnabSicoob extends LeitorCnab
{
    const CODIGO_COMPENSACAO_SICOOB = "756";
    const SEGMENTO_A = "A";
    const SEGMENTO_J = "J";
    const SEGMENTO_B = "B";
    const ID_REGISTRO_HEADER = "0";
    const ID_REGISTRO_LOTE = "1";
    const ID_REGISTRO_TRANSACAO = "3";
    const ID_REGISTRO_TRAILLER_LOTE = "5";
    const ID_REGISTRO_TRAILLER = "9";

    const TIPO_MOVIMENTO_INCLUSAO = 0;
    const TIPO_MOVIMENTO_ALTERACAO = 5;
    const TIPO_MOVIMENTO_EXCLUSAO = 9;

    private $tpProcessamento;
    private $cdagencia;
    private $contcorr;
    private $dtvencimento;
    private $vrpago;
    private $nossonumero;
    private $dtpagamento;
    private $vrdesconto;
    private $vracrescimo;
    private $nmrazsocforn;
    private $ocorrHeader;
    private $nrlanbol = 0;
    private $cdfilial = 0;
    private $contLote = 0;

    public function __construct($arquivo, $cdempresa, $cdbanco)
    {
        try
        {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
        }
        catch(\Exception $e)
        {
            throw $e;
        }
    }

    /**
     *  @param $linha
     * @return bool
     */
    private function validaArquivo($linha)
    {
        parent::setStatusArquivo(true);
        $idReg = substr($linha, 7, 1);
        if ($this->contlinha == 1 and $idReg != self::ID_REGISTRO_HEADER)
        {
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.", true);
            throw new Exception("Erro de identificação do registro header.", 1);
            parent::setStatusArquivo(false);
        }
        if ($this->contlinha > 1 and !in_array($idReg, [self::ID_REGISTRO_HEADER, self::ID_REGISTRO_LOTE, self::ID_REGISTRO_TRANSACAO, self::ID_REGISTRO_TRAILLER_LOTE, self::ID_REGISTRO_TRAILLER, ]))
        {
            self::gravaErros($this->contlinha, "Identificação do registro não é válida", true);
            throw new Exception("Identificação do registro não é válida.", 1);

            parent::setStatusArquivo(false);
        }
        return parent::getStatusArquivo();
    }
    
    public function leArquivo($nrorg)
    {
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){
                    if (strlen($linha) > 0){
                        $this->retorno = new Retorno();

                        // Busca o Tipo de Registro do Arquivo
                        $idRegistro = substr($linha,7,1);
                        if(self::validaArquivo($linha)){
                            switch ($idRegistro) {
                                case(self::ID_REGISTRO_HEADER):
                                    self::leHeaderArquivo($linha);
                                    break;

                                case(self::ID_REGISTRO_TRANSACAO):
                                    $this->contLote++;
                                    self::leRegistroDetalhe($linha, $nrorg, $this->contLote);
                                    break;

                                case(self::ID_REGISTRO_LOTE):
                                    self::leRegistroLote($linha);
                                    break;

                                case(self::ID_REGISTRO_TRAILLER_LOTE):
                                    self::leRegistroTraillerLote($linha);
                                    break;
                            }
                        }
                        $this->retornos[] = $this->retorno;
                    }
                }
            }
            parent::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function leHeaderArquivo($header){

      // CAMPO: Código do BCO na compensação
      // POSIÇÃO: 1 a 3
      // PICTURE: 9(3)
      if (substr($header, 0, 3) != self::CODIGO_COMPENSACAO_SICOOB){
          self::gravaErros($this->contlinha, "Código do banco inválido.",true);
          parent::setStatusArquivo(false);
      }

      // CAMPO: Tipo de inscrição da empresa
      // POSIÇÃO: 18 a 18
      // PICTURE: 9(1)
      $idInsc = trim(substr($header, 17, 1));
      if (empty($idInsc)){
          self::gravaErros($this->contlinha, "Tipo de incrição da empresa não informado",true);
          parent::setStatusArquivo(false);
      }

      // CAMPO: Código Remessa/Retorno
      // POSIÇÃO: 143 a 143
      // PICTURE: 9(1)
      if(substr($header, 142, 1) != '2'){
          self::gravaErros($this->contlinha, "Arquivo de retorno inválido.", true);
          parent::setStatusArquivo(false);
      }
        
    }
    
    private function leRegistroLote($registro){
      
      // CAMPO: Número da agência centralizadora
      // POSIÇÃO: 53 a 57
      // PICTURE: 9(4)
      $this->cdagencia = substr($registro, 52, 5);

      // CAMPO: Conta debitada
      // POSIÇÃO: 59 a 70
      // PICTURE: 9(9)
      $this->contcorr = substr($registro, 58, 13);
      
      // CAMPO: Ocorrências
      // POSIÇÃO: 231 a 240
      // PICTURE: X(10)
      $this->ocorrHeader = substr($registro, 229, 10);
    }
    
    
  private function leRegistroDetalhe($registro, $nrorg, $contadorLote){
    $segmento = substr($registro,13,1);
    $nrlanbol = $this->nrlanbol;
    $cdFilial = $this->cdfilial;
    $nossonumero = $this->nossonumero;
    $cdcontcorr = $this->contcorr;
    $agencia =  $this->cdagencia;
    $vrpago = $this->vrpago;
    $vrdesconto = $this->vrdesconto;
    $vracrescimo = $this->vracrescimo;
    $nmrazsocforn = $this->nmrazsocforn;
    $dtpagamento = $this->dtpagamento;
    $dtvencimento = $this->dtvencimento;

    if($segmento == 'A'){
        $this->segmentoA($registro);
    } elseif($segmento == 'B'){
        $this->segmentoB($registro);
    } elseif($segmento == 'J'){
        if(substr($registro, 17,2) == '52'){
            $this->segmentoJ52($registro);
        } else {
            $this->segmentoJ($registro);
        }
    } 
    
    if($contadorLote == 2){
        $this->retorno->setCdfilial($cdFilial);
        $this->retorno->setNrlanbolpag($nrlanbol);
        $this->retorno->setBoleto('BOLETO');
        $this->retorno->setCdconta($cdcontcorr);
        $this->retorno->setCdagencia($agencia);
        $this->retorno->setVrpago($vrpago);
        $this->retorno->setVrpago($vrpago);
        $this->retorno->setCdlinha(str_pad($this->contlinha - 1,'0',STR_PAD_LEFT));
        $this->retorno->setDtpagamento($dtpagamento);
        $this->retorno->setNmrazsocforn($nmrazsocforn);
        $this->retorno->setCdtpbaixapag(self::getTipoBaixaPagamento($cdFilial, $nrlanbol, $nrorg));
        $this->retorno->setCdreducont(self::getReducont($cdFilial, $nrlanbol, $nrorg));
        $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($cdFilial, $nrlanbol, $nrorg);
        $valorFinalPagoBoleto = ($boletpagar) ? $boletpagar->getVrfinalbpag() : 0;
        $this->retorno->setVrfinalpago($valorFinalPagoBoleto);
        $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$this->ocorrencia.$this->ocorrHeader));
    }
  }
    
  private function segmentoA($registro){
      $this->cdfilial = substr($registro, 73, 4);
      $this->nrlanbol = substr($registro, 77, 10);
      $this->nossonumero = substr($registro, 134, 20);
      $this->dtpagamento = substr($registro, 93, 8);
      $vrpago = floatval(substr($registro, 119, 15));
      $this->vrpago =(round($vrpago / 100, 2));
      $this->nmrazsocforn = substr($registro, 43,30);
      $nrRetorno = substr($registro, 229, 10);
      $this->ocorrencia = substr($registro,229,10);
      if (empty($nrRetorno)){
          self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
          parent::setStatusArquivo(false);
      }
  }
  
  private function segmentoB($registro){
      $this->retorno->setDtvencimento(substr($registro, 127, 8));
      $vrdesconto = floatval(substr($registro, 150, 15));
      $this->retorno->setVrdesconto(round($vrdesconto / 100, 2));
      $vracrescimo = floatval(substr($registro, 180, 13));
      $this->retorno->setVracrescimo(floatval(substr($registro, 195, 13)));
      $this->contLote = 0;
  }
  
  private function segmentoJ($registro){
       $this->cdfilial = substr($registro, 182, 4);
       $this->nrlanbol = substr($registro, 186, 10);
       
       $this->dtvencimento = substr($registro,91,8);
       
       $this->dtpagamento = substr($registro, 144, 8);
       
       $vrpago = floatval(substr($registro, 152, 15));
       $this->vrpago =(round($vrpago / 100, 2));
       
       $vrdesconto = floatval(substr($registro, 114, 15));
       $this->vrdesconto =  round($vrdesconto / 100, 2);
       
       $vracrescimo = floatval(substr($registro, 129, 15));
       $this->vracrescimo = round($vracrescimo / 100,2);
       
       $this->nossonumero = substr($registro, 202, 20);
       
       $this->nmrazsocforn = substr($registro, 61,30);
       
       $nrRetorno = substr($registro, 229, 10);
       $this->ocorrencia = substr($registro,229,10);
       if (empty($nrRetorno)){
           self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
           parent::setStatusArquivo(false);
       }
  }
  
  private function segmentoJ52($registro){
      $this->retorno->setDtvencimento($this->dtvencimento);
      $this->retorno->setVrdesconto($this->vrdesconto);
      $this->retorno->setVracrescimo($this->vracrescimo);
      $this->contLote = 0;
  }
  
  private function leRegistroTraillerLote($registro){
      // CAMPO: Ocorrências
      // POSIÇÃO: 231 a 240
      // PICTURE: X(10)
      $ocorrTraillerLote = trim(substr($registro, 229, 10));
      $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$ocorrTraillerLote.$this->ocorrHeader)); 
  }


  /**
   * @param $cdocorrencia
   * @return string
  */
  private function situacaoCnab($cdocorrencia){
    if (strpos($cdocorrencia,'BD') !== false){
      return '20'; // Retorno Ok (Agendado)
    }elseif (strpos($cdocorrencia,'00') !== false){
      return '90'; // Baixado via CNAB
    }elseif((strpos($cdocorrencia,'BD') === false) || (strpos($cdocorrencia,'00') === false) || (strpos($cdocorrencia,'AJ') === false)){
        return '01'; // Retorno com erro
    } else {
        return '10';
    }  
  }

}

