<?php
namespace Contabilidade\Repositories;

use \Contabilidade\Entities\Lanclote as Item;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use \Setforms2\Shared\Connection;
use \Setforms2\Shared\Registry;

class Lanclote extends \Doctrine\ORM\EntityRepository
{
    public static function retornaSeqLote($nrorg, $cdempresa, $cdlote){

        $sql = "SELECT MAX(NRSEQLANLOTE) AS SEQUENCIAL
                  FROM LANCLOTE
                 WHERE CDEMPRESA = '$cdempresa'
                   AND CDLOTE    = '$cdlote'
                   AND NRORG     = '$nrorg'";
        try {
            return Registry::getEntityManager()->getConnection()->fetchColumn($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function retornaNumeroslancamentos($nrorg, $cdempresa, $cdlote, $periodoini, $periodofin){

        $sql = "SELECT DISTINCT NRLANCLOTE
                  FROM LANCLOTE
                 WHERE CDEMPRESA = '$cdempresa'
                   AND CDLOTE    = '$cdlote'
                   AND NRORG     = '$nrorg'
                   AND DTLANCLOTE NOT BETWEEN '$periodoini' AND '$periodofin'";
        try {
            return Registry::getEntityManager()->getConnection()->fetchAll($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function confereIdComercial($nrorg, $cdempresa, $cdlote){

        $sql = "SELECT L.NRLANCLOTE, L.CDCONTCTBL, L.NRSEQLANLOTE
                  FROM LANCLOTE L, CONTCTBL C
                 WHERE (L.CDEMPRESA  = '$cdempresa')
                   AND (L.CDLOTE     = '$cdlote')
                   AND (L.NRORG      = '$nrorg')
                   AND (C.CDCONTCTBL = L.CDCONTCTBL)
                   AND (C.NRORG      = L.NRORG)
                   AND (C.IDTIPOCOME IN ('C', 'F', 'B'))
                   AND (L.CDLLOTCOME IS NULL)";
        try {
            return Registry::getEntityManager()->getConnection()->fetchAll($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function confereIdLancamento($nrorg, $cdempresa, $cdlote){
        $sql = "SELECT L.CDLOTE, L.NRLANCLOTE,
                       L.CDCONTCTBL, L.IDLLOTCOME,
                       L.CDLLOTCOME, L.NRSEQLANLOTE
                  FROM LANCLOTE L, CONTCTBL C
                 WHERE (L.NRORG      = '$nrorg')
                   AND (L.CDEMPRESA  = '$cdempresa')
                   AND (L.CDLOTE     = '$cdlote')
                   AND (L.CDCONTCTBL = C.CDCONTCTBL)
                   AND (L.NRORG      = C.NRORG)
                   AND (C.IDTIPOCOME IS NULL)
                   AND (L.IDLLOTCOME IS NOT NULL)";
        try {
            return Registry::getEntityManager()->getConnection()->fetchAll($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function confereCdFornecedor($nrorg, $cdempresa, $cdlote){
        $sql = "SELECT CDLOTE, NRLANCLOTE, CDCONTCTBL, CDLLOTCOME, NRSEQLANLOTE
                  FROM LANCLOTE
                 WHERE (NRORG     = '$nrorg')
                   AND (CDEMPRESA = '$cdempresa')
                   AND (CDLOTE    = '$cdlote')
                   AND (CDLLOTCOME <> '' OR CDLLOTCOME IS NOT NULL)
                   AND (IDLLOTCOME = 'F')
                   AND (NOT EXISTS (SELECT CDFORNECED
                                      FROM FORNECEDOR
                                     WHERE(CDFORNECED = CDLLOTCOME)))";
        try {
            return Registry::getEntityManager()->getConnection()->fetchAll($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function confereCdCliente($nrorg, $cdempresa, $cdlote){
        $sql = "SELECT CDLOTE, NRLANCLOTE, CDCONTCTBL, CDLLOTCOME
                  FROM LANCLOTE
                 WHERE (NRORG     = '$nrorg')
                   AND (CDEMPRESA = '$cdempresa')
                   AND (CDLOTE    = '$cdlote')
                   AND (CDLLOTCOME <> '' OR CDLLOTCOME IS NOT NULL)
                   AND (IDLLOTCOME = 'C')
                   AND (NOT EXISTS (SELECT CDCLIENTE
                                      FROM CLIENTE
                                     WHERE(CDCLIENTE = CDLLOTCOME)))";
        try {
            return Registry::getEntityManager()->getConnection()->fetchAll($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function confereCdBanco($nrorg, $cdempresa, $cdlote){
        $sql = "SELECT CDLOTE, NRLANCLOTE, CDCONTCTBL, CDLLOTCOME
                  FROM LANCLOTE
                 WHERE (NRORG     = '$nrorg')
                   AND (CDEMPRESA = '$cdempresa')
                   AND (CDLOTE    = '$cdlote')
                   AND (CDLLOTCOME <> '' OR CDLLOTCOME IS NOT NULL)
                   AND (IDLLOTCOME = 'B')
                   AND (NOT EXISTS (SELECT CDCONTCORR
                                      FROM CONTCORR
                                     WHERE(CDREDUCONT = CDLLOTCOME)))";
        try {
            return Registry::getEntityManager()->getConnection()->fetchAll($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function existeCapaLoteCad($nrorg, $cdempresa, $cdlote){
        $sql = "SELECT COUNT(DISTINCT NRLANCLOTE) AS QTLANC
                  FROM LANCLOTE
                 WHERE (NRORG     = '$nrorg')
                   AND (CDEMPRESA = '$cdempresa')
                   AND (CDLOTE    = '$cdlote')";
        try {
            return Registry::getEntityManager()->getConnection()->fetchAll($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * DCU002 - Gerar Lote Contábil
     * Insere um registro na tabela Lanclote
     *
     * @param Item $item
     * @return boolean
     * @throws \Exception
     */
    public static function insereLanclote(Item $item)
    {
        try {
            $params = array(
                'P_CDEMPRESA' => $item->getCdempresa()?$item->getCdempresa():"",
                'P_CDLOTE' => $item->getCdlote()?$item->getCdlote():"",
                'P_NRLANCLOTE' => $item->getNrlanclote()?$item->getNrlanclote():"",
                'P_NRSEQLANLOTE' => $item->getNrseqlanlote(),
                'P_DTLANCLOTE' => $item->getDtlanclote()?$item->getDtlanclote():"",
                'P_CDCONTCTBL' => $item->getCdcontctbl()?$item->getCdcontctbl():"",
                'P_CDHISTPADR' => ($item->getCdhistpadr()?$item->getCdhistpadr():''),
                'P_VRLANCLOTE' => $item->getVrlanclote()?$item->getVrlanclote():"",
                'P_IDFCONTLANCLOTE' => $item->getIdfcontlanclote()?$item->getIdfcontlanclote():"",
                'P_IDTPLANCLOTE' => $item->getIdtplanclote()?$item->getIdtplanclote():"",
                'P_NRORG' => Ambiente::getNrorgTrabalho(),
                'P_DTINCLUSAO' => DateUtil::getDataAtual()->format('d/m/Y')
            );
            $sql = <<<SQL
INSERT INTO LANCLOTE(
	CDEMPRESA,
	CDLOTE,
	NRLANCLOTE,
	NRSEQLANLOTE,
	DTLANCLOTE,
	CDCONTCTBL,
	CDHISTPADR,
	VRLANCLOTE,
	IDFCONTLANCLOTE,
	IDTPLANCLOTE,
	NRORG
) VALUES (
	:P_CDEMPRESA,
	:P_CDLOTE,
	:P_NRLANCLOTE,
	:P_NRSEQLANLOTE,
	:P_DTLANCLOTE,
	:P_CDCONTCTBL,
	:P_CDHISTPADR,
	:P_VRLANCLOTE,
	:P_IDFCONTLANCLOTE,
	:P_IDTPLANCLOTE,
	:P_NRORG
)
SQL;

            $resultado = Connection::getInstance()->execute($sql, $params);
            if ($resultado) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function conferePeriodoCapaLote($nrorg, $cdempresa, $cdlote){
        $sql = "SELECT DISTINCT NRLANCLOTE
                  FROM CAPALOTE, LANCLOTE
                 WHERE (LANCLOTE.CDEMPRESA = '$cdempresa') AND
                       (LANCLOTE.CDLOTE    = '$cdlote')    AND
                       (LANCLOTE.NRORG    = '$nrorg')      AND
                       (CAPALOTE.CDEMPRESA = LANCLOTE.CDEMPRESA)   AND
                       (CAPALOTE.CDLOTE = LANCLOTE.CDLOTE)   AND
                       (CAPALOTE.NRORG = LANCLOTE.NRORG)   AND
                       (NOT LANCLOTE.DTLANCLOTE BETWEEN CAPALOTE.DTINICLOTE AND CAPALOTE.DTFINALOTE)";
        try {
            return Registry::getEntityManager()->getConnection()->fetchColumn($sql);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}